#version 330 core
#define MAX_NUM_TOTAL_LIGHTS 3

layout (location = 0) out vec4 fragColor;
layout (location = 1) out vec4 brightColor;

in vec3 P;
in vec3 N;
in vec2 textureCoord;

in mat3 localSurface2World;

uniform vec3 cameraPosition;

uniform sampler2D textureSampler;

uniform vec3 materialColor = vec3(1.0,1.0,1.0);

//lighting
uniform vec3 ambientColor = vec3(1.0,1.0,1.0);
uniform float ambientIntensity = 0.1;

uniform int pointLightCount = 0;
uniform vec3 pointLightPositions[MAX_NUM_TOTAL_LIGHTS];
uniform float pointLightIntensities[MAX_NUM_TOTAL_LIGHTS];
uniform float pointLightAttenuation[MAX_NUM_TOTAL_LIGHTS];
uniform int pointLightShadowMapBitmask;

uniform float shininess = 20;

//Shadow Mapping
uniform samplerCube depthMap;
uniform vec2 near_far;

//Normal Mapping
uniform sampler2D normalMapSampler;

vec3 sampleOffsetDirections[20] = vec3[]
(
   vec3( 1,  1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1,  1,  1), 
   vec3( 1,  1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1,  1, -1),
   vec3( 1,  1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1,  1,  0),
   vec3( 1,  0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1,  0, -1),
   vec3( 0,  1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0,  1, -1)
);

float Shadow(vec3 fragPos, vec3 lightPos)
{
	vec3 fragToLight = fragPos - lightPos;
	float currentDepth = length(fragToLight);

	// PCF
    float shadow = 0.0;
	float bias = 0.15;
	int samples = 20;

	float viewDistance = length(cameraPosition - fragPos);
	float diskRadius = (near_far.x + (viewDistance / near_far.y)) / near_far.y;  

	for(int i = 0; i < samples; ++i)
	{
		float closestDepth = texture(depthMap, fragToLight + sampleOffsetDirections[i] * diskRadius).r;
		closestDepth *= near_far.y;   // Undo mapping [0;1]
		closestDepth += near_far.x;
		if(currentDepth - bias > closestDepth)
			shadow += 1.0;
	}
	shadow /= float(samples); 

	return shadow;
}

void main() 
{
	vec3 color = texture(textureSampler, textureCoord).rgb;

	vec3 normal = texture(normalMapSampler, textureCoord).rgb;
	normal = normalize(normal * 2.0 - 1.0);
	normal = normalize(localSurface2World * normal);

	vec3 viewDirection = normalize(cameraPosition - P);

	vec3 lightPosition;
	float lightIntensity;
	float lightAttenuation;

	vec3 lightDirection;
	vec3 ray;
	float dist;

    float attenuation;
	vec3 diffuse;
	vec3 specular;
	float shadowfac;

	vec3 totalLighting = color * ambientIntensity;

	// for all light sources
	for (int index = 0; index < pointLightCount && index < MAX_NUM_TOTAL_LIGHTS; index++) 
	{
		lightPosition = pointLightPositions[index];
		lightIntensity = pointLightIntensities[index];
		lightAttenuation = pointLightAttenuation[index];

		ray = lightPosition - P;
		dist = length(ray);
		lightDirection = normalize(ray);
		
		attenuation = 1.0 / (1.0 + lightAttenuation * dist * dist);

		diffuse = color * lightIntensity * max(0.0, dot(normal,lightDirection));

		//---calculate shadow---
		shadowfac = 0.0;
		if(((pointLightShadowMapBitmask >> index) & 1) == 1)
		{
			shadowfac = Shadow(P, lightPosition);
		}

		specular = vec3(0,0,0);
		if (dot(normal, lightDirection) > 0.0 && shadowfac == 0.0) 
		{
			specular = color * lightIntensity * pow(max(0.0, dot(reflect(-lightDirection, normal), viewDirection)), shininess);
		}

		totalLighting += attenuation* ( diffuse + specular ) * (1.0 - shadowfac);
	}

	fragColor = vec4(totalLighting, 1.0);

	float brightness = dot(fragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
	if(brightness > 0.65)
		brightColor = fragColor;
}